﻿## Copyright (c) 2012, Brother John, http://encodingwissen.de/
## All rights reserved.
## 
## Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met: 
## 
## 1. Redistributions of source code must retain the above copyright notice, this
##    list of conditions and the following disclaimer. 
## 2. Redistributions in binary form must reproduce the above copyright notice,
##    this list of conditions and the following disclaimer in the documentation
##    and/or other materials provided with the distribution. 
## 
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
## DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
## ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys
import os.path
import subprocess

# =============== User config ================

# Calls/paths to the required external tools. Valid is every form that can be used
# to call the tools from the command line.
# Do not change the enclosing quotation – the r''. Never use additional quotes, not even
# if paths contain spaces.
## 32bit x264 encoder. http://x264.nl/
encCmd32 = r'x264'
## 64bit x264 encoder. Ignored on 32bit Windows, encCmd32 will be used instead.
encCmd64 = r'x264_x64'
## MuldeR's logging tool. http://code.google.com/p/mulder/downloads/list?q=logger
loggerCmd = r'logger'
## Avs2yuv. Only needed on 64 bit Windows, ignored on 32bit Windows.
## http://akuvian.org/src/avisynth/avs2yuv/
avs2yuvCmd = r'avs2yuv'

# Default values:
## Default x264 options
x264DefaultCli = "--preset slow --tune film"
## Default destination format: .mkv, .mp4 or .264
outputExt = ".mkv"
## Default pixel aspect ratio (PAR, SAR) written as x:y
sar = "1:1"
## Default constant rate factor
crf = 20

# ============ End of user config ============

infile = ""
outfile = ""
idxfile = ""


def ShowHelpMsg():
    print(
"""x264 encoding script v2.0
Encodes video with x264 in 1-pass CRF mode. Logs information about x264 version,
encoding configuration and x264’s info output to encstats.txt.

Usage: venc [--mkv|--mp4|--264] infile [sar] [crf] [more options]
Example: venc --mp4 "D:\Video files\Source.avs" 16:11 19 --preset fast

The order of the parameters is important! 
Only the [more options] section is passed to x264 as is.

Output formats
    Can be omitted (defaults to %s) or one of the following:
    --mkv   Matroska container
    --mp4   MP4 container
    --264   raw H.264 stream

infile
    AviSynth script or any other valid x264 input file.
    On 64bit Windows if infile is an avs script 64bit x264 via avs2yuv is used
    by default. Call the script as venc32 to force usage of x264 32bit.
    If infile is not an avs script 64bit x264 is used automatically.
    On 32bit Windows obviously 64bit is never used.

The output file is created in the same folder as infile with "-ENCODED" added
to its name. Existing output files are overwritten.

The three x264 config sections are optional. But if you specify one of them
you must specifiy all preceding ones as well.

[sar]
    Pixel aspect ratio in x264 compatible notation. Defaults to %s.

[crf]
    CRF quality level. Defaults to %d.

[more options]
    Additional x264 options. Defaults to:
    %s
    Do not use this section to override CRF or SAR!""" % (outputExt, sar, crf, x264DefaultCli))


def CheckBin(execCmd):
    devnull = open(os.devnull, 'w')
    try:
        subprocess.call(execCmd, stdout=devnull, stderr=devnull)
    except OSError:
        sys.exit(execCmd + ": command not found.")
    devnull.close()


def CheckAvs():
    avsDll = os.path.join(os.environ['SystemRoot'], ('SysWOW64' if x64Platform else 'system32'), 'avisynth.dll')
    if not os.path.exists(avsDll):
        sys.exit(avsDll + " not found.\nPlease install 32bit AviSynth from http://avisynth.org/")


def Encode(options, use64bit, useAvs):
    global x264DefaultCli, outputExt, infile, idxfile, outfile, sar, crf
    global encCmd64
    if not use64bit: encCmd64 = encCmd32
    logfile = os.path.join(os.path.dirname(infile), "encstats.txt")

    CheckBin(loggerCmd)
    
    fullCliCmd = ""
    x264Cli = ""
    encoder = ""
    if useAvs and use64bit:
        CheckAvs()
        CheckBin(encCmd64)
        CheckBin(avs2yuvCmd)
        encoder = encCmd64
        x264Cli = '--sar '+sar+ ' --crf '+str(crf)+ ' '+options
        fullCliCmd = '"' + avs2yuvCmd + '" "' + infile + '" - | ' + \
                     '"' + encCmd64 + '" ' + x264Cli + '-o "' + outfile + '" --demuxer y4m -' + r' 2>&1 | ' + \
                     '"' + loggerCmd + '" -log "' + logfile + '" -append -filter " frames, " -format raw -silent : -'
    elif useAvs and (not use64bit):
        CheckAvs()
        CheckBin(encCmd32)
        encoder = encCmd32
        x264Cli = '--sar '+sar+ ' --crf '+str(crf)+ ' '+options
        fullCliCmd = '"' + encCmd32 + '" ' + x264Cli + '-o "' + outfile + '" "' + infile + '"' + r' 2>&1 | ' + \
                     '"' + loggerCmd + '" -log "' + logfile + '" -append -filter " frames, " -format raw -silent : -'
    else: #no avisynth, 64bit x264
        CheckBin(encCmd64)
        CheckBin(loggerCmd)
        encoder = encCmd64
        x264Cli = '--sar '+sar+ ' --crf '+str(crf)+ ' '+options+ '--index "' + idxfile + '" '
        fullCliCmd = '"' + encCmd64 + '" ' + x264Cli + '-o "' + outfile + '" "' + infile + '"' + r' 2>&1 | ' + \
                     '"' + loggerCmd + '" -log "' + logfile + '" -append -filter " frames, " -filter ": indexing input file" -format raw -silent : -'

    print("-------------------------------------------------------------------------------\n"
          "64bit   : "+ ("yes" if use64bit else "no") +"\n"
          "AviSynth: "+ ("yes" if useAvs else "no") +"\n"
          "Input   : "+infile+"\n"
          "Output  : "+outfile+"\n"
          "SAR     : "+sar+"\n"
          "CRF     : "+str(crf)+"\n"
          "Options : "+x264Cli+"\n"
          "-------------------------------------------------------------------------------\n")
    
    os.system('echo x264 64bit: ' + ("yes" if use64bit else "no") + ' > "' + logfile + '"')
    os.system('echo AviSynth  : ' + ("yes" if useAvs else "no") + ' >> "' + logfile + '"')
    os.system('""' + encoder + '" --version >> "' + logfile + '""')
    os.system('echo ' + x264Cli + ' >> "' + logfile + '"')
    os.system('"' + fullCliCmd + '"')


useAvs = True
params = sys.argv

# detect platform "bitness"
x64Platform = os.getenv("PROCESSOR_ARCHITECTURE", "") == "AMD64" or \
              os.getenv("PROCESSOR_ARCHITEW6432", "") == "AMD64"
use64bit = x64Platform

# start cli parsing
if len(params) == 1:
    ShowHelpMsg()
    sys.exit(1)

if x64Platform and (os.path.basename(params[0]) == "venc32.py"):
    use64bit = False

params.pop(0)

# output file format/extension
if params[0] == "--mkv":
    outputExt = ".mkv"
    params.pop(0)
elif params[0] == "--mp4":
    outputExt = ".mp4"
    params.pop(0)
elif params[0] == "--264":
    outputExt = ".264"
    params.pop(0)

if len(params) == 0:
    ShowHelpMsg()
    sys.exit(1)

# in/output file
if os.path.isfile(params[0]):
    infile = os.path.abspath(params[0])
    path, ext = os.path.splitext(infile)
    if ext != ".avs":
        useAvs = False
        use64bit = use64bit and x64Platform
    idxfile = path + ".x264idx"
    outfile = path + "-ENCODED" + outputExt
    params.pop(0)
else:
    sys.exit(params[0] + ": input file not found.")

# SAR
if len(params) != 0:
    sar = params[0]
    params.pop(0)
    
# CRF
if len(params) != 0:
    crf = params[0]
    params.pop(0)

# more options
x264Cli = ""
if len(params) == 0:
    x264Cli = x264DefaultCli + ' '
else:
    for arg in params:
        x264Cli += arg + ' '

Encode(x264Cli, use64bit, useAvs)
